package game;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import game.log.Log;
import game.util.Tuple;
import game.util.Util;

public class ServerProcess {
	private final ProcessBuilder builder;
	
	private Process process;
	private BufferedWriter writer;
	private boolean running;
	private String message;
	private int total;
	private int progress = -1;
	
	public ServerProcess(File dir, int minMem, int maxMem, int port, int distance, byte[] key) {
        StringBuilder sb = new StringBuilder();
        for(int z = 0; z < key.length; z++) {
        	sb.append(String.format("%02x", key[z] & 255));
        }
		this.builder = new ProcessBuilder("java", "-Xms", minMem + "M", "-Xmx", maxMem + "M", "-jar", "game.jar", "game.Server", 
				"-p", "" + port, "-d", "" + distance, "-t", sb.toString()).directory(dir).redirectErrorStream(true);
	}
	
	public void start() {
		this.builder.directory().mkdirs();
		try {
			this.process = this.builder.start();
		}
		catch(IOException e) {
			Log.SYSTEM.error(e, "Konnte Server nicht starten");
			this.process = null;
		}
//		this.builder = null;
		if(this.process == null)
			return;
		Thread con = new Thread(new Runnable() {
			private final BufferedReader reader = new BufferedReader(new InputStreamReader(new BufferedInputStream(ServerProcess.this.process.getInputStream())));
			
			public void run() {
				while(true) {
					String line;
					try {
						line = this.reader.readLine();
					}
					catch(IOException e) {
						line = null;
					}
					if(line == null)
						break;
					if(line.startsWith("#")) {
						line = line.substring(1);
						Tuple<String, String> data = Util.getKeyValue(line);
						if(data.first.equals("running"))
							ServerProcess.this.running = Boolean.parseBoolean(data.second);
						if(data.first.equals("message"))
							ServerProcess.this.message = data.second;
						if(data.first.equals("total"))
							ServerProcess.this.total = Integer.parseInt(data.second);
						if(data.first.equals("progress"))
							ServerProcess.this.progress = Integer.parseInt(data.second);
						continue;
					}
					Log.SYSTEM.info("==SERVER== %s", line);
				}
			}
		}, "Server console listener");
		con.setDaemon(true);
		con.start();
		this.writer = new BufferedWriter(new OutputStreamWriter(new BufferedOutputStream(this.process.getOutputStream())));
	}
	
	public File getFolder() {
		return this.builder.directory();
	}

	public boolean isStarted() {
		return this.running;
	}

	public boolean isStopped() {
		return !this.running;
	}
	
	public String getMessage() {
		return this.message;
	}

	public int getProgress() {
		return this.progress;
	}

	public int getTotal() {
		return this.total;
	}
	
	public void shutdown() {
		try {
			this.writer.write("end\n");
		}
		catch(IOException e) {
		}
	}
}
