package game.biome;

import game.entity.npc.EntityBloodElf;
import game.entity.npc.EntityCultivator;
import game.entity.npc.EntityFireDemon;
import game.entity.npc.EntityMagma;
import game.entity.npc.EntityMetalhead;
import game.entity.npc.EntityTiefling;
import game.init.Blocks;
import game.rng.Random;
import game.rng.WeightedList;
import game.world.BlockPos;
import game.world.WorldServer;
import game.worldgen.FeatureOres;
import game.worldgen.feature.WorldGenFire;
import game.worldgen.feature.WorldGenGlowStone;
import game.worldgen.feature.WorldGenHellLava;
import game.worldgen.foliage.WorldGenMushroom;

public class BiomeHell extends Biome
{
	private final int subtype;
    private final WorldGenFire fireGen;
    private final WorldGenGlowStone glowStoneGen1;
    private final WorldGenGlowStone glowStoneGen2;
    private final FeatureOres quartzGen;
    private final WorldGenHellLava lavaGen1;
    private final WorldGenHellLava lavaGen2;
    private final WorldGenMushroom brownMushroomGen;
    private final WorldGenMushroom redMushroomGen;
	
    public BiomeHell(int id, int subtype)
    {
        super(id);
        this.subtype = subtype;
        if(this.subtype == 0) {
	        this.mobs.add(new RngSpawn(EntityBloodElf.class, 10, 1, 2));
	        this.mobs.add(new RngSpawn(EntityMetalhead.class, 1, 1, 1));
	        this.fireGen = new WorldGenFire();
	        this.glowStoneGen1 = new WorldGenGlowStone();
	        this.glowStoneGen2 = new WorldGenGlowStone();
	        this.quartzGen = new FeatureOres(Blocks.quartz_ore.getState(), 16, 0, 14, 10, 118, false);
	        this.lavaGen1 = new WorldGenHellLava(Blocks.flowing_lava, true);
	        this.lavaGen2 = new WorldGenHellLava(Blocks.flowing_lava, false);
	        this.brownMushroomGen = new WorldGenMushroom(Blocks.brown_mushroom);
	        this.redMushroomGen = new WorldGenMushroom(Blocks.red_mushroom);
        }
        else {
        	this.mobs.add(new RngSpawn(EntityBloodElf.class, 50, 2, 10));
            this.mobs.add(new RngSpawn(EntityCultivator.class, 10, 1, 1));
	        this.fireGen = null;
	        this.glowStoneGen1 = null;
	        this.glowStoneGen2 = null;
	        this.quartzGen = null;
	        this.lavaGen1 = null;
	        this.lavaGen2 = null;
	        this.brownMushroomGen = null;
	        this.redMushroomGen = null;
        }
        if(this.subtype == 2) {
            this.topBlock = Blocks.ash.getState();
            this.fillerBlock = Blocks.rock.getState();
        }
        else {
            this.topBlock = Blocks.hellrock.getState();
            this.fillerBlock = Blocks.hellrock.getState();
        }
    }
    
    protected void addMobs(WeightedList<RngSpawn> mobs) {
    	mobs.add(new RngSpawn(EntityFireDemon.class, 50, 4, 4));
        mobs.add(new RngSpawn(EntityTiefling.class, 100, 4, 4));
        mobs.add(new RngSpawn(EntityMagma.class, 1, 4, 4));
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        if(this.subtype == 0) {
        	for (int i = 0; i < 8; ++i)
            {
                this.lavaGen2.generate(worldIn, rand, pos.add(rand.zrange(16) + 8, rand.zrange(120) + 4, rand.zrange(16) + 8));
            }

            for (int j = 0; j < rand.zrange(rand.zrange(10) + 1) + 1; ++j)
            {
                this.fireGen.generate(worldIn, rand, pos.add(rand.zrange(16) + 8, rand.zrange(120) + 4, rand.zrange(16) + 8));
            }

            for (int k = 0; k < rand.zrange(rand.zrange(10) + 1); ++k)
            {
                this.glowStoneGen1.generate(worldIn, rand, pos.add(rand.zrange(16) + 8, rand.zrange(120) + 4, rand.zrange(16) + 8));
            }

            for (int l = 0; l < 10; ++l)
            {
                this.glowStoneGen2.generate(worldIn, rand, pos.add(rand.zrange(16) + 8, rand.zrange(128), rand.zrange(16) + 8));
            }

            if (rand.chance())
            {
                this.brownMushroomGen.generate(worldIn, rand, pos.add(rand.zrange(16) + 8, rand.zrange(128), rand.zrange(16) + 8));
            }

            if (rand.chance())
            {
                this.redMushroomGen.generate(worldIn, rand, pos.add(rand.zrange(16) + 8, rand.zrange(128), rand.zrange(16) + 8));
            }

//            for (int i1 = 0; i1 < 16; ++i1)
//            {
            this.quartzGen.generate(worldIn, rand, pos); // .add(rand.nextInt(16), rand.nextInt(108) + 10, rand.nextInt(16)));
//            }

            for (int j1 = 0; j1 < 16; ++j1)
            {
                this.lavaGen1.generate(worldIn, rand, pos.add(rand.zrange(16), rand.zrange(108) + 10, rand.zrange(16)));
            }
        }
        else {
            super.decorate(worldIn, rand, pos);
        }
    }
    
//    protected Decorator createBiomeDecorator()
//    {
//        return this.subtype == 0 ? new DecoratorHell() : super.createBiomeDecorator();
//    }

    public int getGrassColorAtPos(BlockPos pos)
    {
        return 0;
    }

    public int getFoliageColorAtPos(BlockPos pos)
    {
    	return 0;
    }
}
