package game.command;

import java.util.Collection;
import java.util.List;

import game.collect.Lists;
import game.entity.Entity;
import game.entity.types.EntityLiving;
import game.world.WorldServer;

public class EntityParser extends PlayerEntityParser {
	protected final boolean livingOnly;
	
	public EntityParser(String name, boolean useSender, boolean livingOnly) {
		super(name, useSender);
		this.livingOnly = livingOnly;
	}

	public Object parse(CommandEnvironment env, String input) {
		Entity entity = null;
		if(input.startsWith("#")) {
			int id = -1;
			try {
				id = Integer.parseInt(input.substring(1));
			}
			catch(NumberFormatException e) {
			}
			if(id != -1) {
				for(WorldServer world : env.getServer().getWorlds()) {
					entity = world.getEntityByID(id);
					if(entity != null)
						break;
				}
			}
		}
		else {
			entity = (Entity)super.parse(env, input);
		}
		if(entity == null)
			throw new RunException("Objekt '%s' wurde nicht gefunden", input);
		else if(this.livingOnly && !(entity instanceof EntityLiving))
			throw new RunException("Objekt '%s' muss lebendig sein", input);
		return entity;
	}
	
	public Object getDefault(CommandEnvironment env) {
		return this.useSender && (this.livingOnly ? (env.getExecutor() instanceof EntityLiving) : (env.getExecutor() instanceof Entity)) ? env.getExecutor() : super.getDefault(env);
	}
	
	public Collection<String> getCompletions(CommandEnvironment env) {
		Entity target = env.getExecutor().getPointedEntity();
		List<String> comp = target == null || (this.livingOnly && !(target instanceof EntityLiving)) ? Lists.newArrayList() : Lists.newArrayList("#" + target.getId());
		comp.addAll(super.getCompletions(env));
		return comp;
	}

	public Class<?> getTypeClass() {
		return this.livingOnly ? EntityLiving.class : Entity.class;
	}
}
