package game.command.commands;

import java.util.List;
import java.util.Map.Entry;
import java.util.function.Function;

import game.collect.Lists;

import game.command.ArgumentParser;
import game.command.CachedExecutable;
import game.command.Parameter;
import game.command.CommandEnvironment;
import game.command.Command;
import game.command.Executor;
import game.util.Util;

public class CommandHelp extends Command {
	public CommandHelp(CommandEnvironment env) {
		super("help");

		this.setParamsOptional();
		this.addEnum("command", CachedExecutable.class, env.getExecutables().values());
	}
	
	public void exec(CommandEnvironment env, Executor exec, CachedExecutable command) {
		if(command == null) {
			for(CachedExecutable cmd : env.getExecutables().values()) {
				this.exec(env, exec, cmd);
			}
			return;
		}
		List<String> list = Lists.newArrayList();
		for(Entry<String, Parameter> entry : command.getParameters().entrySet()) {
			Parameter param = entry.getValue();
			if(entry.getKey().length() == 1 && !param.isPositional() && param.hasShortName()) {
				list.add("-" + param.getShortName() + (param.getParsers().isEmpty() ? " (" + param.getName() + ")" : (param.getParsers().size() == 1 && 
					param.getParsers().get(0).getName().equals(param.getName()) ? " <" + param.getName() + ">" : 
					" (" + param.getName() + ")" + Util.buildLines(" ", new Function<ArgumentParser, String>() {
					public String apply(ArgumentParser parser) {
						return "<" + parser.getName() + ">";
					}
				}, param.getParsers()))));
			}
		}
		for(Parameter param : command.getPositionals()) {
			list.add((param.isRequired() ? "<" : "[") + (param.getParsers().size() == 1 && 
				param.getParsers().get(0).getName().equals(param.getName()) ? param.getName() : 
				"(" + param.getName() + ") " + Util.buildLines(" ", new Function<ArgumentParser, String>() {
				public String apply(ArgumentParser parser) {
					return "<" + parser.getName() + ">";
				}
			}, param.getParsers())) + (param.isRequired() ? ">" : "]"));
		}
		exec.logConsole("%s %s", command.getName(), Util.buildLines(" ", list));
	}
}
