package game.command.commands;

import java.util.List;
import game.command.CommandEnvironment;
import game.command.Command;
import game.command.Executor;
import game.entity.types.EntityLiving;
import game.potion.Potion;
import game.potion.PotionEffect;

public class CommandPotion extends Command {
	public CommandPotion() {
		super("potion");
		
		this.addLivingEntityList("entities", true);
		this.addEnum("type", Potion.class, Potion.values());
		this.setParamsOptional();
		this.addInt("duration", 0, 1000000, 1000000);
		this.addInt("strength", 1, 256, 1);
		
		this.addFlag("particles", 'p');
		this.addFlag("ambient", 'a');
		this.addFlag("keep", 'k');
	}
	
	public Object exec(CommandEnvironment env, Executor exec, List<EntityLiving> entities, Potion type, int duration, int strength, boolean particles, boolean ambient, boolean keep) {
		int done = 0;
		for(EntityLiving entity : entities) {
			if(entity.isPotionApplicable(type)) {
				if(type.isInstant()) {
					type.onImpact(null, null, entity, strength - 1, 1.0);
				}
				else {
					PotionEffect effect = new PotionEffect(type, duration == 0 ? Integer.MAX_VALUE : (duration * 20), strength - 1, ambient, particles);
					if(!keep && entity.hasEffect(type))
						entity.removeEffect(type);
					entity.addEffect(effect);
				}
				if(type.isInstant() || duration == 0)
					exec.logConsole("%d * %s an %s gegeben", strength, type.getDisplay(), entity.getCommandName());
				else
					exec.logConsole("%d * %s für %d Sekunden an %s gegeben", strength, type.getDisplay(), duration, entity.getCommandName());
				done++;
			}
		}
		if(done > 1)
			exec.logConsole("%d Effekte vergeben", done);
		return done;
	}
}
