package game.command.commands;

import java.util.List;
import java.util.Set;

import game.collect.Lists;
import game.collect.Sets;

import game.command.CommandEnvironment;
import game.command.RunException;
import game.command.Command;
import game.command.Executor;
import game.entity.Entity;
import game.entity.types.EntityLiving;
import game.init.EntityRegistry;
import game.nbt.NBTTagCompound;
import game.network.Player;
import game.util.Util;
import game.world.Vec3;
import game.world.WorldServer;

public class CommandSpawn extends Command {
	public CommandSpawn() {
		super("spawn");
		
		Set<String> names = Sets.newTreeSet();
		for(Class<? extends Entity> clazz : EntityRegistry.getAllClasses()) {
			names.add(EntityRegistry.getEntityString(clazz));
		}
		names.add("Lightning");
		this.addEnum("type", String.class, names);
		this.setParamsOptional();
		this.addVector("position", true, true);
		this.addWorld("dim", true);
		this.addTag("tag");
		
		this.addFlag("noinit", 'n');
		this.addInt("count", 'c', 1, 1024, 1);
		this.addTag("postTag", 'p');
	}
	
	public Object exec(CommandEnvironment env, Executor exec, String type, Vec3 pos, WorldServer world, NBTTagCompound tag, boolean noinit, int count, NBTTagCompound postTag) {
		if(type.equalsIgnoreCase("Lightning")) {
			for(int z = 0; z < count; z++) {
				int color = 0xffffff;
				if(tag != null && tag.hasKey("color", 3))
					color = tag.getInteger("color");
				else if(tag != null && tag.hasKey("color", 8)) {
					try {
						color = Integer.parseUnsignedInt(tag.getString("color"), 16);
					}
					catch(NumberFormatException e) {
					}
				}
				world.strikeLightning(pos.xCoord, pos.yCoord, pos.zCoord, color, 
						tag != null && tag.hasKey("damage", 3) ? tag.getInteger("damage") : 0, tag != null && tag.hasKey("fire", 1) && tag.getBoolean("fire"), 
						exec instanceof Player && tag != null && tag.hasKey("summoned", 1) && tag.getBoolean("summoned") ? ((Player)exec).getPresentEntity() : null);
			}
			exec.logConsole("%sBlitz bei %d, %d, %d in %s erschaffen", count == 1 ? "" : (count + "x "), (int)pos.xCoord, (int)pos.yCoord, (int)pos.zCoord, world.dimension.getFormattedName(false));
			return null;
		}
		else {
			List<String> spawned = Lists.newArrayList();
			for(int z = 0; z < count; z++) {
				Entity entity = EntityRegistry.createEntityByName(type, world);
				if(entity == null)
					throw new RunException("Objekt konnte nicht erzeugt werden");
				entity.setLocationAndAngles(pos.xCoord, pos.yCoord, pos.zCoord, world.rand.floatv() * 360.0f, 0.0f);
				if(tag != null) {
					NBTTagCompound ent = new NBTTagCompound();
					entity.writeToNBT(ent);
					ent.merge(tag);
					entity.readFromNBT(ent);
				}
				if(!noinit && (entity instanceof EntityLiving))
					((EntityLiving)entity).onInitialSpawn(null);
				world.spawnEntityInWorld(entity);
				if(postTag != null) {
					NBTTagCompound ent = new NBTTagCompound();
					entity.writeToNBT(ent);
					ent.merge(postTag);
					entity.readFromNBT(ent);
				}
				spawned.add("#" + entity.getId());
			}
			exec.logConsole("%s%s bei %d, %d, %d in %s erschaffen", count == 1 ? "" : (count + "x "), EntityRegistry.getEntityName(type), (int)pos.xCoord, (int)pos.yCoord, (int)pos.zCoord, world.dimension.getFormattedName(false));
			return Util.buildLines(",", spawned);
		}
	}
}
