package game.network;

import game.Server;
import game.color.TextColor;
import game.init.Config;
import game.init.NameRegistry;
import game.log.Log;
import game.packet.LPacketPasswordResponse;
import game.packet.RPacketDisconnect;
import game.rng.Random;

public class LoginHandler extends NetHandler
{
    private static enum LoginState {
        PASSWORD, READY_TO_ACCEPT, ACCEPTED;
    }
    
    private final Server server;
    public final NetConnection netManager;
    
    private LoginState state = LoginState.PASSWORD;
    private int timer;
    private String loginUser;
    private String loginPass;

    public LoginHandler(Server server, NetConnection netManager)
    {
        this.netManager = netManager;
        this.server = server;
    }

    public void closeConnection(String reason)
    {
        try
        {
            Log.JNI.info("Trenne " + this.getConnectionInfo());
            this.netManager.sendPacket(new RPacketDisconnect(reason));
            this.netManager.closeChannel(reason);
        }
        catch (Exception exception)
        {
            Log.JNI.error(exception, "Fehler beim Trennen des Spielers beim Login");
        }
    }
    
    public void onDisconnect(String reason)
    {
        Log.JNI.info(this.getConnectionInfo() + " wurde beim Login getrennt: " + TextColor.stripCodes(reason));
    }

    public String getConnectionInfo()
    {
        return this.loginUser != null ? (this.loginUser + " (" + this.netManager.getCutAddress()
        		+ ")") : this.netManager.getCutAddress();
    }
    
    public void update()
    {
        if(this.state == LoginState.READY_TO_ACCEPT)
        	this.tryAcceptPlayer();
//        else if (this.currentLoginState == LoginState.DELAY_ACCEPT)
//        {
//            if (this.server.getPlayer(this.loginUser) == null)
//            {
//                this.currentLoginState = LoginState.ACCEPTED;
//                this.server.addPlayer(this.networkManager, this.loginUser);
//            }
//        }
        if(this.timer++ == 600)
            this.closeConnection("Anmeldung dauerte zu lange");
    }

    private void tryAcceptPlayer()
    {
		if(this.server.getPlayer(this.loginUser) != null) {
			this.closeConnection("Nutzername '" + this.loginUser + "' ist bereits vergeben.");
			return;
		}
//            if (this.server.getPlayer(this.loginUser) != null)
//            {
//            	this.closeConnection("Nutzername '" + this.loginUser + "' ist bereits vergeben.");
//            	return;
////                this.currentLoginState = LoginState.DELAY_ACCEPT;
////                player.netHandler.kick("Du hast dich von einen anderen Ort verbunden");
//            }
//            this.networkManager.sendPacket(new RPacketLoginSuccess());
        String kick = this.server.addPlayer(this.netManager, this.loginUser, this.loginPass);
        if(kick != null)
        	this.closeConnection(kick);
        else
            this.state = LoginState.ACCEPTED;
    }

//    public void processLoginStart(LPacketLoginStart packetIn)
//    {
//		if(this.state != LoginState.HELLO)
//			throw new IllegalStateException("Unerwartetes Start-Paket");
//    	if(!this.netManager.isLocalChannel()) {
//			this.state = LoginState.PASSWORD;
//        	this.netManager.sendPacket(new RPacketPasswordRequest()); // !Config.password.isEmpty(), Config.auth
//        }
//    }
    
    public void processPasswordResponse(LPacketPasswordResponse packetIn) {
		if(this.state != LoginState.PASSWORD)
			throw new IllegalStateException("Unerwartetes Passwort-Paket");
		this.loginUser = packetIn.getUser();
		if(packetIn.getLocal() != null) {
			// TODO: key verification
			this.netManager.setLocal();
    		this.loginPass = "";
//    		this.loginUser = Config.localUser;
    		if(this.loginUser.length() > Player.MAX_USER_LENGTH || (!this.loginUser.isEmpty() && !Player.isValidUser(this.loginUser))) {
        		Random rand = new Random();
        		do {
        			this.loginUser = NameRegistry.FANTASY.generate(rand, rand.range(2, 4)).toLowerCase();
        		}
        		while(this.loginUser.length() > Player.MAX_USER_LENGTH || !Player.isValidUser(this.loginUser)); // || this.server.getPlayer(this.loginUser) != null);
//        		this.server.setVar("local_user", this.loginUser);
    		}
    		this.server.setLocalUser(this.loginUser);
		}
        else {
    		this.loginPass = packetIn.getPassword();
            if(this.loginUser.isEmpty() || !Player.isValidUser(this.loginUser))
            	throw new IllegalStateException("Ungültiger Nutzername!");
//    		if(!this.checkConnect(packetIn.getAccess()))
//    			return;
        	if(!Config.password.isEmpty() && !Config.password.equals(packetIn.getAccess())) {
        		this.closeConnection("Falsches Zugangspasswort");
        		return;
    		}
        }
		this.state = LoginState.READY_TO_ACCEPT;
	}
}
