package game.packet;

import java.io.IOException;

import game.network.Player;
import game.network.Packet;
import game.network.PacketBuffer;

public class CPacketMessage implements Packet<Player>
{
	private Type type;
    private String message;

    public CPacketMessage()
    {
    }

    public CPacketMessage(Type type, String str)
    {
    	this.type = type;
        this.message = str.length() > type.length ? str.substring(0, type.length) :str;
    }
    
    public void readPacketData(PacketBuffer buf) throws IOException
    {
    	this.type = buf.readEnumValue(Type.class);
        this.message = buf.readStringFromBuffer(this.type.length);
    }
    
    public void writePacketData(PacketBuffer buf) throws IOException
    {
    	buf.writeEnumValue(this.type);
        buf.writeString(this.message);
    }
    
    public void processPacket(Player handler)
    {
        handler.processMessage(this);
    }

    public Type getType()
    {
        return this.type;
    }

    public String getMessage()
    {
        return this.message;
    }
    
    public static enum Type {
    	COMMAND(Player.MAX_CMD_LENGTH),
    	CHAT(Player.MAX_CMD_LENGTH),
    	DISPLAY(Player.MAX_NICK_LENGTH),
    	INFO(Player.MAX_INFO_LENGTH);
    	// , ITEM(30);
    	
    	private final int length;
    	
    	private Type(int length) {
    		this.length = length;
    	}
    }
}
