package game.packet;

import java.awt.image.BufferedImage;
import java.io.IOException;

import game.init.SpeciesRegistry.ModelType;
import game.network.Player;
import game.network.Packet;
import game.network.PacketBuffer;
import game.renderer.texture.EntityTexManager;

public class CPacketSkin implements Packet<Player> {
	private byte[] texture;
	private String character;

	public CPacketSkin() {
	}

	public CPacketSkin(BufferedImage image, String character, ModelType model) {
		if(image == null) {
			this.texture = null;
			this.character = character;
		}
		else {
			int[] img = new int[model.texWidth * model.texHeight];
			image.getRGB(0, 0, image.getWidth(), image.getHeight(), img, 0, image.getWidth());
			this.texture = EntityTexManager.imageToComp(img, model);
			this.character = null;
		}
	}

	public byte[] getCompressed() {
		return this.texture;
	}

	public String getCharacter() {
		return this.character;
	}

	public void readPacketData(PacketBuffer buf) throws IOException {
		if(buf.readBoolean()) {
			this.texture = null;
			this.character = buf.readStringFromBuffer(64);
		}
		else {
			this.texture = buf.readByteArray();
			this.character = null;
			if(this.texture.length == 0 || this.texture.length > EntityTexManager.MAX_SKIN_SIZE)
				this.texture = new byte[EntityTexManager.MAX_SKIN_SIZE];
		}
	}

	public void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeBoolean(this.texture == null);
		if(this.texture == null)
			buf.writeString(this.character);
		else
			buf.writeByteArray(this.texture);
	}

	public void processPacket(Player handler) {
		handler.processSkin(this);
	}
}
