package game.packet;

import java.io.IOException;

import game.network.LoginHandler;
import game.network.Player;
import game.network.Packet;
import game.network.PacketBuffer;

public class LPacketPasswordResponse implements Packet<LoginHandler>
{
    private String user;
    private String access;
    private String password;
    private byte[] local;

    public LPacketPasswordResponse()
    {
    }

    public LPacketPasswordResponse(String userIn, String accessIn, String passwordIn, byte[] local)
    {
        this.user = userIn;
        this.access = accessIn;
        this.password = passwordIn;
        this.local = local;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.user = buf.readStringFromBuffer(Player.MAX_USER_LENGTH);
        this.access = buf.readStringFromBuffer(Player.MAX_PASS_LENGTH);
        this.password = buf.readStringFromBuffer(Player.MAX_PASS_LENGTH);
        this.local = buf.readByteArray();
        this.local = this.local.length == 0 ? null : this.local;
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeString(this.user);
        buf.writeString(this.access);
        buf.writeString(this.password);
        buf.writeByteArray(this.local == null ? new byte[0] : this.local);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(LoginHandler handler)
    {
        handler.processPasswordResponse(this);
    }

    public String getUser()
    {
        return this.user;
    }

    public String getAccess()
    {
        return this.access;
    }

    public String getPassword()
    {
        return this.password;
    }

    public byte[] getLocal()
    {
        return this.local;
    }
}
