package game.packet;

import java.io.IOException;

import game.entity.Entity;
import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;
import game.world.World;

public class S1APacketEntityStatus implements Packet<ClientPlayer>
{
    private int entityId;
    private byte logicOpcode;

    public S1APacketEntityStatus()
    {
    }

    public S1APacketEntityStatus(Entity entityIn, byte opCodeIn)
    {
        this.entityId = entityIn.getId();
        this.logicOpcode = opCodeIn;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readInt();
        this.logicOpcode = buf.readByte();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeInt(this.entityId);
        buf.writeByte(this.logicOpcode);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ClientPlayer handler)
    {
        handler.handleEntityStatus(this);
    }

    public Entity getEntity(World worldIn)
    {
        return worldIn.getEntityByID(this.entityId);
    }

    public byte getOpCode()
    {
        return this.logicOpcode;
    }
}
