package game.packet;

import java.io.IOException;

import game.entity.Entity;
import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;

public class S1BPacketEntityAttach implements Packet<ClientPlayer>
{
    private int leash;
    private int entityId;
    private int vehicleEntityId;

    public S1BPacketEntityAttach()
    {
    }

    public S1BPacketEntityAttach(int leashIn, Entity entityIn, Entity vehicle)
    {
        this.leash = leashIn;
        this.entityId = entityIn.getId();
        this.vehicleEntityId = vehicle != null ? vehicle.getId() : -1;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readInt();
        this.vehicleEntityId = buf.readInt();
        this.leash = buf.readUnsignedByte();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeInt(this.entityId);
        buf.writeInt(this.vehicleEntityId);
        buf.writeByte(this.leash);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ClientPlayer handler)
    {
        handler.handleEntityAttach(this);
    }

    public int getLeash()
    {
        return this.leash;
    }

    public int getEntityId()
    {
        return this.entityId;
    }

    public int getVehicleEntityId()
    {
        return this.vehicleEntityId;
    }
}
