package game.packet;

import java.io.IOException;

import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;
import game.potion.Potion;
import game.potion.PotionEffect;

public class S1DPacketEntityEffect implements Packet<ClientPlayer>
{
    private int entityId;
    private Potion effectId;
    private int amplifier;
    private int duration;
    private int remaining;
    private boolean particles;

    public S1DPacketEntityEffect()
    {
    }

    public S1DPacketEntityEffect(int entityIdIn, PotionEffect effect)
    {
        this.entityId = entityIdIn;
        this.effectId = effect.getPotion();
        this.amplifier = /* (byte)(*/ effect.getAmplifier(); // & 255);

//        if (effect.getDuration() > 32767)
//        {
//            this.duration = 32767;
//        }
//        else
//        {
        this.duration = effect.getDuration();
        this.remaining = effect.getRemaining();
//        }

        this.particles = effect.getIsShowParticles();
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readVarIntFromBuffer();
        this.effectId = buf.readEnumValue(Potion.class);
        this.amplifier = buf.readVarIntFromBuffer();
        this.duration = buf.readVarIntFromBuffer();
        this.remaining = buf.readVarIntFromBuffer();
        this.particles = buf.readBoolean();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVarIntToBuffer(this.entityId);
        buf.writeEnumValue(this.effectId);
        buf.writeVarIntToBuffer(this.amplifier);
        buf.writeVarIntToBuffer(this.duration);
        buf.writeVarIntToBuffer(this.remaining);
        buf.writeBoolean(this.particles);
    }
    
    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ClientPlayer handler)
    {
        handler.handleEntityEffect(this);
    }

    public int getEntityId()
    {
        return this.entityId;
    }

    public Potion getEffectId()
    {
        return this.effectId;
    }

    public int getAmplifier()
    {
        return this.amplifier;
    }

    public int getDuration()
    {
        return this.duration;
    }

    public int getRemaining()
    {
        return this.remaining;
    }

    public boolean hasParticles()
    {
        return this.particles;
    }
}
