package game.packet;

import java.io.IOException;

import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;
import game.util.ExtMath;

public class S2BPacketChangeGameState implements Packet<ClientPlayer>
{
    private Action action;
    private int param;

    public S2BPacketChangeGameState()
    {
    }

    public S2BPacketChangeGameState(Action action)
    {
        this(action, 0);
    }

    public S2BPacketChangeGameState(Action action, float param)
    {
        this(action, (int)(param * 1000.0f));
    }

    public S2BPacketChangeGameState(Action action, int param)
    {
        this.action = action;
        this.param = param;
    }
    
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.action = buf.readEnumValue(Action.class);
        this.param = buf.readVarIntFromBuffer();
    }
    
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeEnumValue(this.action);
        buf.writeVarIntToBuffer(this.param);
    }
    
    public void processPacket(ClientPlayer handler)
    {
        handler.handleChangeGameState(this);
    }

    public Action getAction()
    {
        return this.action;
    }

    public int getInt()
    {
        return this.param < 0 ? 0 : this.param;
    }

    public float getFloat(boolean clamp)
    {
        return ExtMath.clampf((float)this.param / 1000.0f, clamp ? 0.0f : -1000000.0f, clamp ? 1.0f : 1000000.0f);
    }
    
    public static enum Action {
    	SET_WEATHER, RAIN_STRENGTH, DARKNESS, FOG_STRENGTH, TEMPERATURE;
    }
}
