package game.packet;

import java.io.IOException;

import game.init.TileRegistry;
import game.nbt.NBTTagCompound;
import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;
import game.tileentity.TileEntity;
import game.world.BlockPos;

public class S35PacketUpdateTileEntity implements Packet<ClientPlayer>
{
    private BlockPos blockPos;
    private int type;
    private NBTTagCompound nbt;

    public S35PacketUpdateTileEntity()
    {
    }

    public S35PacketUpdateTileEntity(TileEntity tile)
    {
        this.blockPos = tile.getPos();
        this.type = TileRegistry.classToIdMap.get(tile.getClass());
        tile.writeToNBT(this.nbt = new NBTTagCompound());
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.blockPos = buf.readBlockPos();
        this.type = buf.readUnsignedByte();
        this.nbt = buf.readNBTTagCompoundFromBuffer();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeBlockPos(this.blockPos);
        buf.writeByte((byte)this.type);
        buf.writeNBTTagCompoundToBuffer(this.nbt);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ClientPlayer handler)
    {
        handler.handleUpdateTileEntity(this);
    }

    public BlockPos getPos()
    {
        return this.blockPos;
    }

    public boolean isTileEntityType(TileEntity tile)
    {
        return this.type == TileRegistry.classToIdMap.get(tile.getClass());
    }

    public NBTTagCompound getNbtCompound()
    {
        return this.nbt;
    }
}
