package game.packet;

import java.io.IOException;

import game.entity.npc.EntityNPC;
import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;

public class S39PacketPlayerAbilities implements Packet<ClientPlayer> {
	private boolean flying;
	private boolean noClip;

	public S39PacketPlayerAbilities() {
	}

	public S39PacketPlayerAbilities(EntityNPC capabilities) {
		this.flying = capabilities.flying;
		this.noClip = capabilities.noclip;
	}

	public void readPacketData(PacketBuffer buf) throws IOException {
		byte bt = buf.readByte();
		this.flying = (bt & 1) > 0;
		this.noClip = (bt & 2) > 0;
	}

	public void writePacketData(PacketBuffer buf) throws IOException {
		byte bt = 0;

		if(this.isFlying()) {
			bt = (byte)(bt | 1);
		}

		if(this.isNoclip()) {
			bt = (byte)(bt | 2);
		}

		buf.writeByte(bt);
	}

	public void processPacket(ClientPlayer handler) {
		handler.handlePlayerAbilities(this);
	}

	public boolean isFlying() {
		return this.flying;
	}

	public boolean isNoclip() {
		return this.noClip;
	}
}
