package game.packet;

import java.io.IOException;

import game.entity.Entity;
import game.nbt.NBTTagCompound;
import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;
import game.world.World;

public class S43PacketUpdateEntityNBT implements Packet<ClientPlayer>
{
    private int entityId;
    private NBTTagCompound tagCompound;

    public S43PacketUpdateEntityNBT()
    {
    }

    public S43PacketUpdateEntityNBT(int entityIdIn, NBTTagCompound tagCompoundIn)
    {
        this.entityId = entityIdIn;
        this.tagCompound = tagCompoundIn;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readVarIntFromBuffer();
        this.tagCompound = buf.readNBTTagCompoundFromBuffer();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVarIntToBuffer(this.entityId);
        buf.writeNBTTagCompoundToBuffer(this.tagCompound);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(ClientPlayer handler)
    {
        handler.handleEntityNBT(this);
    }

    public NBTTagCompound getTagCompound()
    {
        return this.tagCompound;
    }

    public Entity getEntity(World worldIn)
    {
        return worldIn.getEntityByID(this.entityId);
    }
}
