package game.packet;

import java.io.IOException;

import game.dimension.Dimension;
import game.nbt.NBTTagCompound;
import game.network.ClientPlayer;
import game.network.Packet;
import game.network.PacketBuffer;

public class SPacketJoinGame implements Packet<ClientPlayer> {
	private int entityId;
	private NBTTagCompound dimension;
    private int type;
    private boolean editor;

	public SPacketJoinGame() {
	}

	public SPacketJoinGame(int entityIdIn, Dimension dimensionIn, int type, boolean editor) {
		this.entityId = entityIdIn;
		this.dimension = dimensionIn.toNbt(false);
		this.type = type;
		this.editor = editor;
	}

	public void readPacketData(PacketBuffer buf) throws IOException {
		this.entityId = buf.readInt();
		this.dimension = buf.readNBTTagCompoundFromBuffer();
        this.type = buf.readUnsignedShort();
        this.editor = buf.readBoolean();
	}

	public void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeInt(this.entityId);
		buf.writeNBTTagCompoundToBuffer(this.dimension);
        buf.writeShort(this.type & 65535);
        buf.writeBoolean(this.editor);
	}

	public void processPacket(ClientPlayer handler) {
		handler.handleJoinGame(this);
	}

	public int getEntityId() {
		return this.entityId;
	}

	public Dimension getDimension() {
		return Dimension.getByNbt(this.dimension, false);
	}

    public int getEntityType() {
        return this.type;
    }

    public boolean isInEditor() {
        return this.editor;
    }
}
