package game.renderer.blockmodel;

import java.util.List;

import game.collect.Lists;
import game.collect.Maps;

import game.model.ModelBakery;
import game.model.ModelRotation;
import game.renderer.Vector3f;
import game.renderer.texture.TextureMap;
import game.world.Facing;

public class ModelBlock {
	private final List<BlockPart> elements;
    private final boolean gui3d;
    private final String primary;
    private final String[] layers;
    private final ModelBlock parent;
    
    private boolean occlusion;
	private ModelRotation rotation;
	private boolean uvLock;
    private Transforms transform;
    
    private BlockPart lastPart;
    private BlockPartFace[] last;
    
    
    public ModelBlock slab(boolean upper) {
    	return this.slab(upper, this.primary, this.primary);
    }
    
    public ModelBlock slab(boolean upper, String du) {
    	return this.slab(upper, du, du);
    }
    
    public ModelBlock slab(boolean upper, String down, String up) {
    	return this.add(0, upper ? 8 : 0, 0, 16, upper ? 16 : 8, 16).nswe().uv(0, upper ? 0 : 8, 16, upper ? 8 : 16)
    			.d(down).cull(upper ? null : Facing.DOWN).u(up).cull(upper ? Facing.UP : null);
    }
    
    public ModelBlock vslab(Facing dir) {
    	return this.vslab(dir, this.primary, this.primary);
    }
    
    public ModelBlock vslab(Facing dir, String du) {
    	return this.vslab(dir, du, du);
    }
    
    public ModelBlock vslab(Facing dir, String down, String up) {
    	return this.add(0, 0, 0, 16, 16, 8).du().uv(0, 8, 16, 16).we().uv(0, 8, 16, 16).rot(90).n(down).s(up).noCull().rotate(ModelRotation.getNorthRot(dir));
    }
    
    public ModelBlock cross() {
    	return this.noOcclude()
    			.add(0.8f, 0f, 8f, 15.2f, 16f, 8f).noShade().rotate(8, 8, 8, Facing.Axis.Y, 45, true)
    			.ns().uv(0, 0, 16, 16).noCull()
    			.add(8f, 0f, 0.8f, 8f, 16f, 15.2f).noShade().rotate(8, 8, 8, Facing.Axis.Y, 45, true)
    			.we().uv(0, 0, 16, 16).noCull();
    }
    
    public ModelBlock crossTint() {
    	return this.noOcclude()
    			.add(0.8f, 0f, 8f, 15.2f, 16f, 8f).noShade().rotate(8, 8, 8, Facing.Axis.Y, 45, true)
    			.ns().uv(0, 0, 16, 16).noCull().tint()
    			.add(8f, 0f, 0.8f, 8f, 16f, 15.2f).noShade().rotate(8, 8, 8, Facing.Axis.Y, 45, true)
    			.we().uv(0, 0, 16, 16).noCull().tint();
    }
    
    public ModelBlock stairs(boolean upper, boolean inner, boolean outer, boolean left, Facing dir, String down, String up) {
    	this.add(0, 0, 0, 16, 8, 16).d(down).uv(0, 0, 16, 16).u(up).noCull().uv(0, 0, 16, 16).nswe().uv(0, 8, 16, 16);
    	if(outer)
        	this.add(8, 8, 8, 16, 16, 16).d(down).uv(8, 0, 16, 8).u(up).uv(8, 8, 16, 16).n().noCull().uv(0, 0, 8, 8).s().uv(8, 0, 16, 8)
    				.w().noCull().uv(8, 0, 16, 8).e().uv(0, 0, 8, 8);
    	else
    		this.add(8, 8, 0, 16, 16, 16).d(down).uv(inner ? 8 : 0, 0, 16, 16).u(up).uv(inner ? 8 : 0, 0, 16, 16).n().uv(0, 0, 8, 8)
    				.s().uv(8, 0, 16, 8).w().noCull().uv(0, 0, 16, 8).e().uv(0, 0, 16, 8);
    	if(inner)
    		this.add(0, 8, 8, 8, 16, 16).d(down).uv(0, 0, 8, 8).u(up).uv(0, 8, 8, 16).n().noCull().uv(8, 0, 16, 8).s().uv(0, 0, 8, 8)
					.w().uv(8, 0, 16, 8).e().uv(0, 0, 8, 8);
    	return this.rotate(ModelRotation.getEastRot(left ? Facing.getHorizontal(dir.getHorizontalIndex() + 3) : dir, upper)).uvLock();
    }
    
    
    
    public ModelBlock noOcclude() {
    	this.occlusion = false;
    	return this;
    }
    
    public ModelBlock uvLock() {
    	this.uvLock = true;
    	return this;
    }
    
    public ModelBlock rotate(ModelRotation rot) {
    	this.rotation = rot;
    	return this;
    }
    
    public ModelBlock add(float x1, float y1, float z1, float x2, float y2, float z2) {
    	this.elements.add(this.lastPart = new BlockPart(new Vector3f(x1, y1, z1), new Vector3f(x2, y2, z2), 
    			Maps.newEnumMap(Facing.class), null, true));
    	return this;
    }
    
    public ModelBlock add() {
    	return this.add(0, 0, 0, 16, 16, 16);
    }
    
    public ModelBlock noShade() {
    	this.lastPart.shade = false;
    	return this;
    }
    
    public ModelBlock rotate(float x, float y, float z, Facing.Axis axisIn, float angleIn, boolean rescaleIn) {
    	this.lastPart.partRotation = new BlockPartRotation(x, y, z, axisIn, angleIn, rescaleIn);
    	return this;
    }
    
    public ModelBlock face(String texture, Facing ... faces) {
    	texture = !texture.equals(TextureMap.LOCATION_MISSING_TEXTURE) && texture.indexOf('/') == -1 ? "blocks/" + texture : texture;
    	this.last = new BlockPartFace[faces.length];
    	for(int z = 0; z < faces.length; z++) {
	    	this.lastPart.mapFaces.put(faces[z], this.last[z] = 
	    			new BlockPartFace(faces[z], -1, texture, new BlockFaceUV(this.lastPart.getFaceUvs(faces[z]), 0)));
    	}
    	return this;
    }
    
    public ModelBlock d(String textureIn) {
    	return this.face(textureIn, Facing.DOWN);
    }
    
    public ModelBlock u(String textureIn) {
    	return this.face(textureIn, Facing.UP);
    }
    
    public ModelBlock n(String textureIn) {
    	return this.face(textureIn, Facing.NORTH);
    }
    
    public ModelBlock s(String textureIn) {
    	return this.face(textureIn, Facing.SOUTH);
    }
    
    public ModelBlock w(String textureIn) {
    	return this.face(textureIn, Facing.WEST);
    }
    
    public ModelBlock e(String textureIn) {
    	return this.face(textureIn, Facing.EAST);
    }
    
    public ModelBlock nswe(String textureIn) {
    	return this.face(textureIn, Facing.NORTH, Facing.SOUTH, Facing.WEST, Facing.EAST);
    }
    
    public ModelBlock dnswe(String textureIn) {
    	return this.face(textureIn, Facing.DOWN, Facing.NORTH, Facing.SOUTH, Facing.WEST, Facing.EAST);
    }
    
    public ModelBlock du(String textureIn) {
    	return this.face(textureIn, Facing.DOWN, Facing.UP);
    }
    
    public ModelBlock ns(String textureIn) {
    	return this.face(textureIn, Facing.NORTH, Facing.SOUTH);
    }
    
    public ModelBlock we(String textureIn) {
    	return this.face(textureIn, Facing.WEST, Facing.EAST);
    }
    
    public ModelBlock all(String textureIn) {
    	return this.face(textureIn, Facing.DOWN, Facing.UP, Facing.NORTH, Facing.SOUTH, Facing.WEST, Facing.EAST);
    }
    
    public ModelBlock d() {
    	return this.d(this.primary);
    }
    
    public ModelBlock u() {
    	return this.u(this.primary);
    }
    
    public ModelBlock n() {
    	return this.n(this.primary);
    }
    
    public ModelBlock s() {
    	return this.s(this.primary);
    }
    
    public ModelBlock w() {
    	return this.w(this.primary);
    }
    
    public ModelBlock e() {
    	return this.e(this.primary);
    }
    
    public ModelBlock nswe() {
    	return this.nswe(this.primary);
    }
    
    public ModelBlock dnswe() {
    	return this.dnswe(this.primary);
    }
    
    public ModelBlock du() {
    	return this.du(this.primary);
    }
    
    public ModelBlock ns() {
    	return this.ns(this.primary);
    }
    
    public ModelBlock we() {
    	return this.we(this.primary);
    }
    
    public ModelBlock all() {
    	return this.all(this.primary);
    }
    
    public ModelBlock cull(Facing cull) {
    	for(BlockPartFace last : this.last) {
    		last.cull = cull;
    	}
    	return this;
    }
    
    public ModelBlock noCull() {
    	return this.cull(null);
    }
    
    public ModelBlock tint() {
    	for(BlockPartFace last : this.last) {
    		last.tint = 0;
    	}
    	return this;
    }
    
    public ModelBlock rot(int rot) {
    	for(BlockPartFace last : this.last) {
        	last.uv = new BlockFaceUV(last.uv.uvs, rot);
    	}
    	return this;
    }
    
    public ModelBlock uv(float x1, float y1, float x2, float y2) {
    	for(BlockPartFace last : this.last) {
        	last.uv = new BlockFaceUV(new float[] {x1, y1, x2, y2}, last.uv.rotation);
    	}
    	return this;
    }
    
    
    public ModelBlock(String primary) {
    	this(null, Lists.newArrayList(), primary != null && primary.indexOf('/') == -1 ? "blocks/" + primary : primary, true, true, Transforms.DEFAULT, null);
    }

    public ModelBlock(String primary, Transforms transform, String ... layers) {
    	this(ModelBakery.MODEL_GENERATED, ModelBakery.MODEL_GENERATED.elements, 
    			primary.indexOf('/') == -1 ? "items/" + primary : primary, false, false, transform, layers);
    }
    
    public ModelBlock(Transforms transform, String ... layers) {
    	this(ModelBakery.MODEL_GENERATED, ModelBakery.MODEL_GENERATED.elements, 
    			layers[0].indexOf('/') == -1 ? "items/" + layers[0] : layers[0], false, false, transform, layers);
    }
    
    protected ModelBlock(String primary, List<BlockPart> elements, Transforms transform, String ... layers) {
    	this(null, elements, primary, false, false, transform, layers);
    }

    public ModelBlock(ModelBlock parent, Transforms transform) {
        this(parent, Lists.newArrayList(), parent.getPrimary(), false, true, transform, null);
    }

	private ModelBlock(ModelBlock parent, List<BlockPart> elements, String primary, boolean occlude, boolean gui3d, 
			Transforms transform, String[] layers) {
		for(int z = 0; layers != null && z < layers.length; z++) {
			layers[z] = layers[z].indexOf('/') == -1 ? "items/" + layers[z] : layers[z];
		}
		this.elements = parent == null ? elements : parent.getElements();
		this.occlusion = parent == null ? occlude : parent.isAmbientOcclusion();
		this.gui3d = gui3d;
		this.primary = primary == null ? TextureMap.LOCATION_MISSING_TEXTURE : primary;
		this.parent = parent;
		this.transform = transform;
		this.uvLock = false;
		this.rotation = ModelRotation.X0_Y0;
		this.layers = layers;
	}

	public List<BlockPart> getElements() {
		return this.elements;
	}

	public boolean isAmbientOcclusion() {
		return this.occlusion;
	}

	public boolean isGui3d() {
		return this.gui3d;
	}

	public String getPrimary() {
		return this.primary;
	}

	public String[] getTextures() {
		return this.layers;
	}

	public int getNumTextures() {
		return this.layers == null ? 0 : this.layers.length;
	}

	public String getTexture(int layer) {
		return this.layers == null || this.layers[layer] == null ? TextureMap.LOCATION_MISSING_TEXTURE : this.layers[layer];
	}

	public ModelBlock getParent() {
		return this.parent;
	}

	public Transforms getTransform() {
		return this.transform;
	}

	public ModelRotation getRotation() {
		return this.rotation;
	}

	public boolean isUvLocked() {
		return this.uvLock;
	}
}
